﻿using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.Util;
using System.Collections.Generic;

namespace Example_ConnectionComponent1
{
    public class BoltParameter
    {
        public enum SoltInMode
        {
            NONE = 0,
            BEAM = 1,
            PLATE = 2,
            BOTH = 3,
        }

        public Vec3 BoltPos = Vec3.ZERO;
        public double BoltSizeX = 0.0;
        public double BoltSizeY = 0.0;
        public Vec3 BoltDirX = Vec3.AXIS_X;
        public Vec3 BoltDirY = Vec3.AXIS_Y;
        public double BoltTopMargin = 0.0;
        public double BoltBottomMargin = 0.0;
        public double BoltRightMargin = 0.0;

        public Define.BoltGroupShape BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_NONE;
        public int HoriCount = 0;
        public int VertCount = 0;
        public List<double> StrideX = new List<double>();
        public List<double> StrideY = new List<double>();
        public List<int> BoltIgnores = new List<int>();
        public int BoltOrientation = 0;
        public List<Vec3> CornerPoints = new List<Vec3>();

        public string BoltStandard;
        public double BoltSize = 0.0;
        public double HoleTol = 0.0;
        public bool ThreadInMaterial = false;
        public Define.PartMadeType MadeType0 = Define.PartMadeType.PART_MADE_SITE;
        public Define.PartMadeType MadeType1 = Define.PartMadeType.PART_MADE_SITE;

        public Define.BoltHoleType HoleType = Define.BoltHoleType.BOLT_HOLE_OVERSIZED;
        public double HoleX = 0.0;
        public double HoleY = 0.0;
        public Define.BoltRotateSlot RotSlot = 0;
        public SoltInMode SlotIn = SoltInMode.NONE;

        public bool PartWasherUp = false;
        public bool PartWasherDown0 = false;
        public bool PartWasherDown1 = false;
        public bool PartNut0 = false;
        public bool PartNut1 = false;

        public double ExtLength = 0.0;
        public double CutLength = 200;
        public int[] BoltDel = new int[0];
        public double[] Bolt_Xintervals = new double[0];
        public double[] Bolt_Yintervals = new double[0];
        public uint SectFlags = 0;

        public BoltParameter()
        {

        }

        public void UpdateCornerPoints()
        {
            CornerPoints.Clear();
            CornerPoints.Add(BoltPos);
            CornerPoints.Add(BoltPos + BoltDirX * BoltSizeX);
            CornerPoints.Add(BoltPos + BoltDirY * BoltSizeY);
            CornerPoints.Add(CornerPoints[1] + BoltDirY * BoltSizeY);
        }

        public void CopyFromBasicProps(DObject basicProps)
        {
            if (basicProps == null)
                return;

            BoltStandard = basicProps.GetString("bolt_standard");
            BoltSize = basicProps.GetDouble("bolt_size");
            HoleTol = basicProps.GetDouble("hole_tol");
            ThreadInMaterial = basicProps.GetInt("thread_in_mtrl") == 2;
            MadeType0 = ParseMadeType(basicProps.GetInt("made_type0"));
            MadeType1 = ParseMadeType(basicProps.GetInt("made_type1"));

            /// FIXME: 自适应孔容差范围，有待转移到底层选项中
            if (HoleTol == double.MaxValue)
            {
                if (BoltSize >= 27.0)
                    HoleTol = 3.0;
                else if (BoltSize >= 16.0)
                    HoleTol = 2.0;
                else
                    HoleTol = 1.5;
            }
        }

        public void CopyFromHoleProps(DObject holeProps)
        {
            if (holeProps == null)
                return;

            HoleX = holeProps.GetDouble("hole_x");
            HoleY = holeProps.GetDouble("hole_y");
            // 孔类型
            {
                int comboIndex = holeProps.GetInt("hole_type");
                switch (comboIndex)
                {
                    case 1:
                        HoleType = Define.BoltHoleType.BOLT_HOLE_SLOTTED;
                        break;
                    case 2:
                    default:
                        HoleType = Define.BoltHoleType.BOLT_HOLE_OVERSIZED;
                        break;
                }
            }
            //
            {
                int comboIndex = holeProps.GetInt("rot_slot");
                switch (comboIndex)
                {
                    case 0:
                        RotSlot = Define.BoltRotateSlot.BOLT_SLOT_ODD;
                        break;
                    case 1:
                        RotSlot = Define.BoltRotateSlot.BOLT_SLOT_EVEN;
                        break;
                    case 2:
                    default:
                        RotSlot = Define.BoltRotateSlot.BOLT_SLOT_PARALLEL;
                        break;
                }
            }
            //
            {
                int comboIndex = holeProps.GetInt("slot_in");
                switch (comboIndex)
                {
                    case 0:
                        SlotIn = SoltInMode.BOTH;
                        break;
                    case 1:
                        SlotIn = SoltInMode.NONE;
                        break;
                    case 2:
                        SlotIn = SoltInMode.BEAM;
                        break;
                    case 3:
                        SlotIn = SoltInMode.PLATE;
                        break;
                    default:
                        SlotIn = SoltInMode.BOTH;
                        break;
                }
            }
        }

        public void CopyFromContentProps(DObject contentProps)
        {
            if (contentProps == null)
                return;
            PartWasherUp = contentProps.GetBool("part_washer_up");
            PartWasherDown0 = contentProps.GetBool("part_washer_down0");
            PartWasherDown1 = contentProps.GetBool("part_washer_down1");
            PartNut0 = contentProps.GetBool("part_nut0");
            PartNut1 = contentProps.GetBool("part_nut1");
        }

        public void CopyFromExtendProps(DObject extendProps)
        {
            if (extendProps == null)
                return;
            ExtLength = extendProps.GetDouble("ext_length");
        }
        public void ApplyToBoltGroup(BoltGroup bolt_group)
        {
            if (bolt_group == null) return;
            // 
            bolt_group.BoltStandard = BoltStandard;
            bolt_group.BoltSize = BoltSize;
            bolt_group.HoleTol = HoleTol;
            bolt_group.ThreadInMtrl = ThreadInMaterial;
            bolt_group.MadeType = MadeType1;
            //
            bolt_group.HoleX = HoleX;
            bolt_group.HoleY = HoleY;
            bolt_group.HoleType = HoleType;
            bolt_group.RotateSlot = RotSlot;
            bolt_group.SectionFlags = SectFlags;
            bolt_group.CutLength = CutLength;
            //
            bolt_group.ExtLength = ExtLength;
            bolt_group.IgnoreIndices = BoltDel;
            bolt_group.Shape = BoltShape;
            bolt_group.ArrayStrideX = Bolt_Xintervals;
            bolt_group.ArrayStrideY = Bolt_Yintervals;
            bolt_group.Refresh();
        }
        private Define.PartMadeType ParseMadeType(int comboIndex)
        {
            switch (comboIndex)
            {
                case 2:
                    return Define.PartMadeType.PART_MADE_WORKSHOP;
                case 1:
                default:
                    return Define.PartMadeType.PART_MADE_SITE;
            }
        }

    }
}
