﻿using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Example_ConnectionComponent1
{

    /// <summary>
    /// 组件上下文类，该类对象用来存储组件公用变量、属性
    /// </summary>
    public class ComponentInstanceContext
    {
        private ConnectionComponent m_comp;
        //主次零件
        private Beam m_mainPart = null;

        private Beam m_subPart = null;

        //方向向量
        private Vec3 normal_top;

        private Vec3 normal_front;

        private Vec3 normal_right;

        //面
        private Plane m_AlignPlane = null;  //切割面

        private Plane subplane_top = null;

        private Plane subplane_bottom = null;

        private Plane subplane_front = null;

        private Plane subplane_back = null;

        private readonly Plane subplane_head = null;

        private readonly Plane subplane_tail = null;

        private Plane mainplane_top = null;

        private Plane mainplane_bottom = null;

        private Plane mainplane_front = null;

        private Plane mainplane_back = null;

        private BeamPlanes m_mainPlanes;

        private BeamPlanes m_subPlanes;

        //其他
        private double sub_height;

        private double sub_width;

        private double main_height;

        private double main_width;

        private Plane projectplane;

        private static List<WeldProperties> m_weldProps;

        private readonly SteelBeam end_part;

        private Vec3 top_point;

        private Vec3 bottom_point;
        ComponentInstanceContext()
        {

        }
        public static ComponentInstanceContext CreateInstance(ConnectionComponent comp, ParamsHandler paramsHandler)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (!compContext.Initialize(comp))
            {
                compContext.Clear();
                return null;
            }
            //焊缝获取属性
            m_weldProps = new List<WeldProperties>();
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                DObject weldobj = paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldobj = WeldPropertyPanel.ParseNativeProps(weldobj);
                WeldProperties weldProps = new WeldProperties();
                weldProps.SetProps(weldobj);
                m_weldProps.Add(weldProps);
            }
            return compContext;
        }

        public ConnectionComponent Comp { get => m_comp; set => m_comp = value; }
        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }
        public Beam SubPart { get => m_subPart; set => m_subPart = value; }
        public Vec3 Normal_top { get => normal_top; set => normal_top = value; }
        public Vec3 Normal_front { get => normal_front; set => normal_front = value; }
        public Vec3 Normal_right { get => normal_right; set => normal_right = value; }
        public BeamPlanes MainPlanes { get => m_mainPlanes; set => m_mainPlanes = value; }
        public BeamPlanes SubPlanes { get => m_subPlanes; set => m_subPlanes = value; }
        public Plane ProjectPlane { get => projectplane; set => projectplane = value; }
        public Plane SubPlane_top { get => subplane_top; set => subplane_top = value; }
        public Plane SubPlane_bottom { get => subplane_bottom; set => subplane_bottom = value; }
        public Plane SubPlane_front { get => subplane_front; set => subplane_front = value; }
        public Plane SubPlane_back { get => subplane_back; set => subplane_back = value; }
        public Plane MainPlane_top { get => mainplane_top; set => mainplane_top = value; }
        public Plane MainPlane_bottom { get => mainplane_bottom; set => mainplane_bottom = value; }
        public Plane MainPlane_front { get => mainplane_front; set => mainplane_front = value; }
        public Plane MainPlane_back { get => mainplane_back; set => mainplane_back = value; }
        public Plane AlignPlane { get => m_AlignPlane; set => m_AlignPlane = value; }
        public double Sub_Height { get => sub_height; set => sub_height = value; }
        public double Sub_Width { get => sub_width; set => sub_width = value; }
        public double Main_Height { get => main_height; set => main_height = value; }
        public double Main_Width { get => main_width; set => main_width = value; }
        public SteelBeam Beam_Part { get; set; } //直梁
        public SteelPlate Plate_Patr { get; set; } //多边形板
        public List<PartBase> Bolt_Connection { get; set; } // 螺栓连接零件

        public bool Initialize(ConnectionComponent comp)
        {
            if (comp == null)
                return false;
            Beam mainPart = comp.Main as Beam;
            Beam subPart1 = comp.Subs.Length > 0 ? comp.Subs.First<TFObject>() as Beam : null;
            Beam subPart2 = comp.Subs.Length > 1 ? comp.Subs[1] as Beam : null;
            if (mainPart == null || subPart1 == null)
                return false;
            m_comp = comp;
            m_mainPart = mainPart;
            m_subPart = subPart1;
            m_mainPlanes = BeamPlanes.CreateFromBeam(m_mainPart);
            m_subPlanes = BeamPlanes.CreateFromBeam(m_subPart);
            ParsePlanes();
            return true;
        }

        ///焊缝三种创建方式
        public Weld CreateWeld(string usage, PartBase main, PartBase sub, Define.SpaceAxis autoWeldPos, int index)
        {
            if (m_weldProps[index].TypeUp == Define.WeldType.WELD_TYPE_NONE && m_weldProps[index].TypeDown == Define.WeldType.WELD_TYPE_NONE)
                return null;
            if (sub == null || main == null)
                return null;
            var weld = m_comp.GetOrCreateAutoWeld(usage, main, sub, autoWeldPos);
            weld.WorkAxis = new Mat43(Normal_top, Normal_right, Normal_front, Vec3.ZERO);
            m_weldProps[index].ApplyToWeld(weld);
            weld.Refresh();
            return weld;
        }
        public Weld CreateWeld(string usage, PartBase main, PartBase sub, Vec3 direction, int index)
        {
            if (m_weldProps[index].TypeUp == Define.WeldType.WELD_TYPE_NONE && m_weldProps[index].TypeDown == Define.WeldType.WELD_TYPE_NONE)
                return null;
            if (sub == null || main == null)
                return null;
            var weld = m_comp.GetOrCreateAutoWeld(usage, main, sub, direction);
            weld.WorkAxis = new Mat43(Normal_top, Normal_right, Normal_front, Vec3.ZERO);
            m_weldProps[index].ApplyToWeld(weld);
            weld.Refresh();
            return weld;
        }
        public void CreateWeld(string usage, PartBase main, PartBase sub, Vec3[] ctrlPoints, int index)
        {
            if (m_weldProps[index].TypeUp == Define.WeldType.WELD_TYPE_NONE && m_weldProps[index].TypeDown == Define.WeldType.WELD_TYPE_NONE)
                return;
            if (sub == null || main == null)
                return;
            var weld = m_comp.GetOrCreatePolyWeld(usage, main, sub, ctrlPoints);
            m_weldProps[index].ApplyToWeld(weld);
            weld.Refresh();
        }
        ///

        /// <summary>
        /// 解析正副零件面，设置方向向量
        /// </summary>
        private void ParsePlanes()
        {
            // 解析主零件各个面
            {
                Vec3 s_point = SubPart.CtrlPoints[0].WorldPos / 2 + SubPart.CtrlPoints[1].WorldPos / 2;
                double angle = MainPlanes.TopBound.Normal.Normalize().Dot(SubPlanes.HeadBound.Normal.Normalize());
                if (angle >= Math.Cos(Math.PI / 4) || angle <= Math.Cos(Math.PI * 3 / 4))
                {//主零件的近次零件面，在顶底面中产生
                    if (MainPlanes.TopBound.SignedDist(s_point) > 0)
                    {//近面为顶面
                        MainPlane_top = MainPlanes.TopBound.Clone();
                        MainPlane_bottom = MainPlanes.BottomBound.Clone();
                        MainPlane_front = MainPlanes.FrontBound.Clone();
                        MainPlane_back = MainPlanes.BackBound.Clone();
                    }
                    else
                    {//近面为底面
                        MainPlane_top = MainPlanes.BottomBound.Clone();
                        MainPlane_bottom = MainPlanes.TopBound.Clone();
                        MainPlane_front = MainPlanes.BackBound.Clone();
                        MainPlane_back = MainPlanes.FrontBound.Clone();
                    }
                }
                else
                {//主零件的近次零件面，在前后面中产生
                    if (MainPlanes.FrontBound.SignedDist(s_point) > 0)
                    {//近面为前面
                        if (MainPlanes.FrontWeb != null)
                            MainPlane_top = MainPlanes.FrontWeb.Clone();
                        else
                            MainPlane_top = MainPlanes.FrontBound.Clone();
                        if (MainPlanes.BackWeb != null)
                            MainPlane_bottom = MainPlanes.BackWeb.Clone();
                        else
                            MainPlane_bottom = MainPlanes.BackBound.Clone();
                        MainPlane_front = MainPlanes.TopBound.Clone();
                        MainPlane_back = MainPlanes.BottomBound.Clone();
                    }
                    else
                    {//近面为后面
                        if (MainPlanes.FrontWeb != null)
                            MainPlane_bottom = MainPlanes.FrontWeb.Clone();
                        else
                            MainPlane_bottom = MainPlanes.FrontBound.Clone();
                        if (MainPlanes.BackWeb != null)
                            MainPlane_top = MainPlanes.BackWeb.Clone();
                        else
                            MainPlane_top = MainPlanes.BackBound.Clone();
                        MainPlane_front = MainPlanes.BottomBound.Clone();
                        MainPlane_back = MainPlanes.TopBound.Clone();
                    }
                }
            }
            // 解析次零件相对次零件的远近侧边界面
            {
                if (SubPart.RotationSide == Define.ObjectSide.SIDE_TOP)
                {
                    switch (Comp.CompDirUp)
                    {
                        case Define.SpaceAxis.AXIS_Z:
                        case Define.SpaceAxis.AXIS_X:
                        case Define.SpaceAxis.AXIS_NX:
                            SubPlane_top = SubPlanes.TopBound.Clone();
                            SubPlane_bottom = SubPlanes.BottomBound.Clone();
                            SubPlane_front = SubPlanes.FrontBound.Clone();
                            SubPlane_back = SubPlanes.BackBound.Clone();
                            break;
                        case Define.SpaceAxis.AXIS_Y:
                            SubPlane_top = SubPlanes.BackBound.Clone();
                            SubPlane_bottom = SubPlanes.FrontBound.Clone();
                            SubPlane_front = SubPlanes.TopBound.Clone();
                            SubPlane_back = SubPlanes.BottomBound.Clone();
                            break;
                        case Define.SpaceAxis.AXIS_NY:

                            SubPlane_top = SubPlanes.FrontBound.Clone();
                            SubPlane_bottom = SubPlanes.BackBound.Clone();
                            SubPlane_front = SubPlanes.BottomBound.Clone();
                            SubPlane_back = SubPlanes.TopBound.Clone();
                            break;
                        case Define.SpaceAxis.AXIS_NZ:
                            SubPlane_top = SubPlanes.BottomBound.Clone();
                            SubPlane_bottom = SubPlanes.TopBound.Clone();
                            SubPlane_front = SubPlanes.BackBound.Clone();
                            SubPlane_back = SubPlanes.FrontBound.Clone();
                            break;
                        default:
                            SubPlane_top = SubPlanes.TopBound.Clone();
                            SubPlane_bottom = SubPlanes.BottomBound.Clone();
                            SubPlane_front = SubPlanes.FrontBound.Clone();
                            SubPlane_back = SubPlanes.BackBound.Clone();
                            break;
                    }
                }
                if (SubPart.RotationSide == Define.ObjectSide.SIDE_FRONT)
                {
                    switch (Comp.CompDirUp)
                    {
                        case Define.SpaceAxis.AXIS_Z:
                            SubPlane_top = SubPlanes.FrontBound.Clone();
                            SubPlane_bottom = SubPlanes.BackBound.Clone();
                            SubPlane_front = SubPlanes.BottomBound.Clone();
                            SubPlane_back = SubPlanes.TopBound.Clone();
                            break;
                        case Define.SpaceAxis.AXIS_Y:
                        case Define.SpaceAxis.AXIS_X:
                        case Define.SpaceAxis.AXIS_NX:
                            SubPlane_top = SubPlanes.TopBound.Clone();
                            SubPlane_bottom = SubPlanes.BottomBound.Clone();
                            SubPlane_front = SubPlanes.FrontBound.Clone();
                            SubPlane_back = SubPlanes.BackBound.Clone();
                            break;
                        case Define.SpaceAxis.AXIS_NY:

                            SubPlane_top = SubPlanes.BottomBound.Clone();
                            SubPlane_bottom = SubPlanes.TopBound.Clone();
                            SubPlane_front = SubPlanes.BackBound.Clone();
                            SubPlane_back = SubPlanes.FrontBound.Clone();
                            break;
                        case Define.SpaceAxis.AXIS_NZ:
                            SubPlane_top = SubPlanes.BackBound.Clone();
                            SubPlane_bottom = SubPlanes.FrontBound.Clone();
                            SubPlane_front = SubPlanes.TopBound.Clone();
                            SubPlane_back = SubPlanes.BottomBound.Clone();
                            break;
                        default:
                            SubPlane_top = SubPlanes.TopBound.Clone();
                            SubPlane_bottom = SubPlanes.BottomBound.Clone();
                            SubPlane_front = SubPlanes.FrontBound.Clone();
                            SubPlane_back = SubPlanes.BackBound.Clone();
                            break;
                    }
                }
                if (SubPart.RotationSide == Define.ObjectSide.SIDE_BOTTOM)
                {
                    switch (Comp.CompDirUp)
                    {
                        case Define.SpaceAxis.AXIS_Z:
                        case Define.SpaceAxis.AXIS_X:
                        case Define.SpaceAxis.AXIS_NX:
                            SubPlane_top = SubPlanes.BottomBound.Clone();
                            SubPlane_bottom = SubPlanes.TopBound.Clone();
                            SubPlane_front = SubPlanes.BackBound.Clone();
                            SubPlane_back = SubPlanes.FrontBound.Clone();
                            break;
                        case Define.SpaceAxis.AXIS_Y:

                            SubPlane_top = SubPlanes.FrontBound.Clone();
                            SubPlane_bottom = SubPlanes.BackBound.Clone();
                            SubPlane_front = SubPlanes.BottomBound.Clone();
                            SubPlane_back = SubPlanes.TopBound.Clone();
                            break;
                        case Define.SpaceAxis.AXIS_NY:
                            SubPlane_top = SubPlanes.BackBound.Clone();
                            SubPlane_bottom = SubPlanes.FrontBound.Clone();
                            SubPlane_front = SubPlanes.TopBound.Clone();
                            SubPlane_back = SubPlanes.BottomBound.Clone();

                            break;
                        case Define.SpaceAxis.AXIS_NZ:
                            SubPlane_top = SubPlanes.TopBound.Clone();
                            SubPlane_bottom = SubPlanes.BottomBound.Clone();
                            SubPlane_front = SubPlanes.FrontBound.Clone();
                            SubPlane_back = SubPlanes.BackBound.Clone();
                            break;
                        default:
                            SubPlane_top = SubPlanes.BottomBound.Clone();
                            SubPlane_bottom = SubPlanes.TopBound.Clone();
                            SubPlane_front = SubPlanes.BackBound.Clone();
                            SubPlane_back = SubPlanes.FrontBound.Clone();
                            break;

                    }
                }
                if (SubPart.RotationSide == Define.ObjectSide.SIDE_BACK)
                {
                    switch (Comp.CompDirUp)
                    {
                        case Define.SpaceAxis.AXIS_Z:
                            SubPlane_top = SubPlanes.BackBound.Clone();
                            SubPlane_bottom = SubPlanes.FrontBound.Clone();
                            SubPlane_front = SubPlanes.TopBound.Clone();
                            SubPlane_back = SubPlanes.BottomBound.Clone();
                            break;
                        case Define.SpaceAxis.AXIS_Y:
                        case Define.SpaceAxis.AXIS_X:
                        case Define.SpaceAxis.AXIS_NX:
                            SubPlane_top = SubPlanes.BottomBound.Clone();
                            SubPlane_bottom = SubPlanes.TopBound.Clone();
                            SubPlane_front = SubPlanes.BackBound.Clone();
                            SubPlane_back = SubPlanes.FrontBound.Clone();
                            break;
                        case Define.SpaceAxis.AXIS_NY:

                            SubPlane_top = SubPlanes.TopBound.Clone();
                            SubPlane_bottom = SubPlanes.BottomBound.Clone();
                            SubPlane_front = SubPlanes.FrontBound.Clone();
                            SubPlane_back = SubPlanes.BackBound.Clone();
                            break;
                        case Define.SpaceAxis.AXIS_NZ:
                            SubPlane_top = SubPlanes.FrontBound.Clone();
                            SubPlane_bottom = SubPlanes.BackBound.Clone();
                            SubPlane_front = SubPlanes.BottomBound.Clone();
                            SubPlane_back = SubPlanes.TopBound.Clone();
                            break;
                        default:
                            SubPlane_top = SubPlanes.BottomBound.Clone();
                            SubPlane_bottom = SubPlanes.TopBound.Clone();
                            SubPlane_front = SubPlanes.BackBound.Clone();
                            SubPlane_back = SubPlanes.FrontBound.Clone();
                            break;
                    }
                }

            }
            //设置整个组件的方向向量
            double top, bottom, front, back, head, tail;
            top = MainPlanes.TopBound.Normal.Normalize().Dot(SubPlane_top.Normal.Normalize());
            bottom = MainPlanes.BottomBound.Normal.Normalize().Dot(SubPlane_top.Normal.Normalize());
            front = MainPlanes.FrontBound.Normal.Normalize().Dot(SubPlane_top.Normal.Normalize());
            back = MainPlanes.BackBound.Normal.Normalize().Dot(SubPlane_top.Normal.Normalize());
            head = MainPlanes.HeadBound.Normal.Normalize().Dot(SubPlane_top.Normal.Normalize());
            tail = MainPlanes.TailBound.Normal.Normalize().Dot(SubPlane_top.Normal.Normalize());
            double[] numbers = new double[] { top, bottom, front, back, head, tail };
            var max = numbers.Max();
            int maxIndex = Array.IndexOf(numbers, max);
            switch (maxIndex)
            {
                case 0:
                    Normal_top = MainPlanes.TopBound.Normal.Normalize();
                    break;
                case 1:
                    Normal_top = MainPlanes.BottomBound.Normal.Normalize();
                    break;
                case 2:
                    Normal_top = MainPlanes.FrontBound.Normal.Normalize();
                    break;
                case 3:
                    Normal_top = MainPlanes.BackBound.Normal.Normalize();
                    break;
                case 4:
                    Normal_top = MainPlanes.HeadBound.Normal.Normalize();
                    break;
                case 5:
                    Normal_top = MainPlanes.TailBound.Normal.Normalize();
                    break;
            }
            Normal_front = MainPlane_bottom.Normal.Normalize();
            Normal_right = Normal_front.Cross(Normal_top).Normalize();
            //设置次零件长宽，和投影平面
            if (Normal_top.IsParallelTo(SubPlanes.TopBound.Normal) || Normal_top.Dot(SubPlanes.TopBound.Normal.Normalize()) > Math.Cos(Math.PI / 2))
            {
                Sub_Height = SubPart.Profile.ShapeHeight;
                Sub_Width = SubPart.Profile.ShapeWidth;
            }
            else
            {
                Sub_Height = SubPart.Profile.ShapeWidth;
                Sub_Width = SubPart.Profile.ShapeHeight;
            }
            ProjectPlane = SubPlane_front.Move(-Sub_Width / 2).Clone();
            //设置次梁的高(倾斜情况)
            double cos = Math.Abs(Normal_top.Dot(SubPlane_top.Normal.Normalize()));
            Sub_Height = Sub_Height / cos;
            //设置主梁的高
            if (MainPlane_top.Normal.IsParallelTo(Normal_front))
            {
                Main_Height = MainPart.Profile.ShapeHeight;
                Main_Width = MainPart.Profile.ShapeWidth;
            }
            else
            {
                Main_Height = MainPart.Profile.ShapeWidth;
                Main_Width = MainPart.Profile.ShapeHeight;
            }
            Bolt_Connection = new List<PartBase>();
        }
        public void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_subPart = null;
            m_mainPlanes = null;
            m_subPlanes = null;
        }
    }
}
