﻿using LoongCAD.UI.Component;
using LoongCAD.Util;
using System;

namespace Example_ConnectionComponent1
{
    public class ParamsHandler : LoongCAD.STC.Plugins.ParamsHandlerBase
    {
        /// <summary>
        /// 属性空值全局总表
        /// </summary>
        /// <remarks><b>此静态变量可直接复制到其他组件代码中使用</b></remarks>
        protected static DObject g_emptyValues = null;

        /// <summary>
        /// 属性内置默认值全局总表
        /// </summary>
        /// <remarks><b>此静态变量可直接复制到其他组件代码中使用</b></remarks>
        protected static DObject g_builtinDefaults = null;

        /// <summary>
        /// 默认构造函数
        /// </summary>
        /// <remarks><b>此方法需要复制到其他组件代码中使用</b></remarks>
        public ParamsHandler() : base(ParamsHandler.EmptyValues, ParamsHandler.BuiltinDefautls) { }

        /// <summary>
        /// 组件公用属性空值表
        /// </summary>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks>
        public static DObject EmptyValues
        {
            get
            {
                if (g_emptyValues == null)
                {
                    IntializeEmptyDefaults();
                }
                return g_emptyValues;
            }
        }
        /// <summary>
        /// 组件公用内置默认值接口
        /// </summary>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks>
        public static DObject BuiltinDefautls
        {
            get
            {
                if (g_builtinDefaults == null)
                {
                    IntializeEmptyDefaults();
                }
                return g_builtinDefaults;
            }
        }
        /// <summary>
        /// 初始化默认值
        /// </summary>
        /// <param name="defaultConfig">从默认值配置加载的默认值</param>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks>
        public override LoongCAD.Util.DObject InitializeDefaultValues(DObject defaultConfig)
        {
            m_defaultValues = new DObject();
            m_defaultValues.CopyFrom(defaultConfig);

            var emptyValues = ParamsHandler.EmptyValues;
            var builtinDefaults = ParamsHandler.BuiltinDefautls;
            if (emptyValues == null || builtinDefaults == null)
            {
                Console.WriteLine("ParamsHandler.InitializeDefaultValues: [Warning] EmptyValue/BuiltinDefaults is null!");
                return m_defaultValues;
            }

            TryCompleteDObjectValues(m_defaultValues, builtinDefaults, emptyValues);
            return m_defaultValues;
        }
        /// <summary>
        /// 初始化组件所有实例公用的属性空值、属性内置默认值
        /// </summary>
        /// <remarks><b>每个组件需要分别实现此方法</b></remarks>
        public static void IntializeEmptyDefaults()
        {
            g_emptyValues = new DObject();
            g_builtinDefaults = new DObject();
            // 板页
            if (true)
            {
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BEAM_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                    PlatePropertyPanel.ParamNames.Thickness, 10.0,
                    PlatePropertyPanel.ParamNames.Width,double.MaxValue,
                    PlatePropertyPanel.ParamNames.Heigth,double.MaxValue,
                    PlatePropertyPanel.ParamNames.NoPrefix, "P",
                    PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                    PlatePropertyPanel.ParamNames.AssNoPrefix, "P",
                    PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                    PlatePropertyPanel.ParamNames.Material, "Q235B",
                    PlatePropertyPanel.ParamNames.Name, "直梁",});
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                    PlatePropertyPanel.ParamNames.Thickness, 10.0,
                    PlatePropertyPanel.ParamNames.Width,double.MaxValue,
                    PlatePropertyPanel.ParamNames.Heigth,200,
                    PlatePropertyPanel.ParamNames.NoPrefix, "P",
                    PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                    PlatePropertyPanel.ParamNames.AssNoPrefix, "P",
                    PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                    PlatePropertyPanel.ParamNames.Material, "Q235B",
                    PlatePropertyPanel.ParamNames.Name, "多边形板",});
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BEAM_PROFILE, BeamPropertyPanel.EmptyValue, (DObject)new object[] {
                    BeamPropertyPanel.ParamNames.Profile, "",
                    BeamPropertyPanel.ParamNames.NoPrefix, "P",
                    BeamPropertyPanel.ParamNames.NoStartPostfix, "1",
                    BeamPropertyPanel.ParamNames.AssNoPrefix, "P",
                    BeamPropertyPanel.ParamNames.AssNoStartPostfix, "1",
                    BeamPropertyPanel.ParamNames.Material, "Q235B",
                    BeamPropertyPanel.ParamNames.Name, "直梁截面",});
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_CUTTER_TYPE, 0, 1);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_ANGLE_Y, double.MaxValue, 10.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_ANGLE_X, double.MaxValue, 10.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_ANGLE, 0, 2);
            }
            // 槽口页
            if (true)
            {
                g_emptyValues.CopyFrom(BeamNotchPage.EmptyValues);
                g_builtinDefaults.CopyFrom(BeamNotchPage.BuiltinDefaults);
            }
            //梁切割页
            if (true)
            {
                g_emptyValues.CopyFrom(BeamCutPage.EmptyValues);
                g_builtinDefaults.CopyFrom(BeamCutPage.BuiltinDefaults);
            }
            // 螺栓
            if (true)
            {
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_BASIC_PROPS, BoltBasicPropertyPanel.EmptyValue, (DObject)new object[] {
                    BoltBasicPropertyPanel.ParamNames.BoltStandard, "TS10.9",
                    BoltBasicPropertyPanel.ParamNames.BoltSize, 20,
                    BoltBasicPropertyPanel.ParamNames.HoleTol,2,
                    BoltBasicPropertyPanel.ParamNames.ThreadInMtrl, 2,
                    BoltBasicPropertyPanel.ParamNames.MadeType0, 2,
                });
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_HOLE_PROPS, BoltHolePanel.EmptyValue, (DObject)new object[] {
                    BoltHolePanel.ParamNames.HoleX, 0.0,
                    BoltHolePanel.ParamNames.HoleY, 0.0,
                    BoltHolePanel.ParamNames.HoleType, 0,
                    BoltHolePanel.ParamNames.RotSlot, 0,
                    BoltHolePanel.ParamNames.SlotIn, 0
                });
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_CONTENT_PROPS, null, (DObject)new object[]
                {
                    BoltContentPanel.ParamNames.PartWasherUp, false,
                    BoltContentPanel.ParamNames.PartWasherDown0, false,
                    BoltContentPanel.ParamNames.PartWasherDown1, true,
                    BoltContentPanel.ParamNames.PartNut0, true,
                    BoltContentPanel.ParamNames.PartNut1, false,
                });
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_EXTEND_PROPS, BoltExtendPanel.EmptyValue, (DObject)new object[]
                {
                    BoltExtendPanel.ParamNames.ExtLength, 0.0,
                });
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_X_BASE, 0, 6);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_X_OFFSET, double.MaxValue, double.MaxValue);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_X_LEFT, double.MaxValue, 40.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_X_RIGHT, double.MaxValue, 40.0);

                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_Y_BASE, 0, 5);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_Y_OFFSET, double.MaxValue, double.MaxValue);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_Y_UP, double.MaxValue, 40.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_Y_DOWN, double.MaxValue, double.MaxValue);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_BOTTOM_OFFSET, double.MaxValue, double.MaxValue);

                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_DELETE_INDICES, new int[] { }, new int[] { });
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_ORIENTATION, 0, 1);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_POS_TYPE, 0, 1);

                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.CUT_LENTH, double.MaxValue, 2000);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_HANG, int.MaxValue, int.MaxValue);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_LIE, int.MaxValue, int.MaxValue);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_X_INTERVALS, new double[] { }, new double[] { });
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_Y_INTERVALS, new double[] { }, new double[] { });
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_DIRECTION, 0, 1);
            }
            // 洞
            if (true)
            {
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HOLE_BASIC_PROPS, BoltBasicPropertyPanel.EmptyValue, (DObject)new object[] {
                    BoltBasicPropertyPanel.ParamNames.BoltStandard, "TS10.9",
                    BoltBasicPropertyPanel.ParamNames.BoltSize,20,
                    BoltBasicPropertyPanel.ParamNames.MadeType0, 2,
                });
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HOLE_POS_TYPE, 0, 0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HOLE_BASE_LEFT_X, double.MaxValue, 40.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HOLE_BASE_RIGHT_X, double.MaxValue, 40.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HOLE_BASE_UP_Y, double.MaxValue, 40.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HOLE_BASE_DOWN_Y, double.MaxValue, 40.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HOLE_SIZE_UP, double.MaxValue, double.MaxValue);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HOLE_SIZE_DOWN, double.MaxValue, double.MaxValue);
            }
            // 焊缝
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WELD_PROPS_LIST[i], WeldPropertyPanel.EmptyValue, new DObject {
                    { WeldPropertyPanel.ParamNames.SizeUp, 6.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeUp, 11 }, // Fillet
                    { WeldPropertyPanel.ParamNames.GrooveAngleUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourUp,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishUp, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapUp, 0.0 },

                    { WeldPropertyPanel.ParamNames.SizeDown, 6.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeDown, 11 }, // Fillet
                    { WeldPropertyPanel.ParamNames.GrooveAngleDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourDown,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishDown, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapDown, 0.0 },

                    { WeldPropertyPanel.ParamNames.EdgeAround, 1 }, // Edge
                    { WeldPropertyPanel.ParamNames.MadeType, 2 }, // Workshop
                    { WeldPropertyPanel.ParamNames.StitchWeld, 1 }, // false
                    { WeldPropertyPanel.ParamNames.RefText, "" }
                });
            }
        }
    }
}
