﻿using Example_ConnectionComponent1.Handlers;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.STC.Plugins.BeamCut;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using System;
using System.Windows.Forms;

namespace Example_ConnectionComponent1
{
    public class PluginMain : ConnectionCompPlugin
    {
        private readonly Guid m_nameGuid = new Guid(System.Reflection.Assembly.GetExecutingAssembly().GetName().Name);
        /// <summary>
        /// 用于开启组件定义基于代码，而非基于软件
        /// </summary>
        public override bool StandAlone => true;
        public override Guid Guid => m_nameGuid;
        public override string CompName => "节点";
        public override string CompGroup => "";
        public override int CompNumber => 000;
        public override string CompDescription => "";
        public override string CompShortDescription => "";
        public override System.Drawing.Image Thumbnail => null;
        public override System.Drawing.Image ButtonImage => null;
        public override System.Drawing.Image SelectedButtonImage => null;
        public override int RequiredSubsMaxCount => 1;
        public override int RequiredSubsMinCount => 1;
#if DEBUG
        public override bool DebugMode => true;
#else
        public override bool DebugMode => false;
#endif
        // 主窗
        private readonly Form_PluginMain m_propsForm = null;
        public PluginMain()
        {
        }
        /// <summary>
        /// 执行组件过程，必需实现此方法
        /// </summary>
        /// <param name="compInst"></param>
        /// <param name="defaultId"></param>
        /// <returns></returns>
        public override bool OnInstanceRun(ComponentBase compInst, long defaultId)
        {
            if (compInst == null || !(compInst is ConnectionComponent))
                return false;

            // 1. 获取组件实例的属性
            DObject compProps = compInst.UIProps;
            if (compProps == null)
                compProps = new DObject();

            // 2. 初始化参数处理器，此过程包含以下操作:
            // * 初始化参数值
            // * 初始化参数空值表
            // * 初始化参数内置默认值表
            // * 组合参数默认值
            ParamsHandler paramsHandler = new ParamsHandler();
            paramsHandler.ParamValues = compProps;
            paramsHandler.InitializeDefaultValues(GetDefaultsData(defaultId));
            // 3. 初始化组件上下文，此过程执行以下操作
            var compContext = ComponentInstanceContext.CreateInstance(compInst as ConnectionComponent, paramsHandler);
            if (compContext == null)
            {
                return false;
            }
            // 4. 执行组件的每一个功能
            {
                //对齐面和切割面示例
                FittingHandler fittingHandler = new FittingHandler(compContext, paramsHandler);
                fittingHandler.Execute();
                //直梁示例
                BeamHandler beamHandler = new BeamHandler(compContext, paramsHandler);
                beamHandler.Execute();
                //多边形板示例
                PlateHandler plateHandler = new PlateHandler(compContext, paramsHandler);
                plateHandler.Execute();
                //螺栓和洞示例
                BoltOrHoleHandler boltOrHoleHandler = new BoltOrHoleHandler(compContext, paramsHandler);
                boltOrHoleHandler.Execute();
                //梁切割示例
                var paraData1 = new BeamCutPage.ParamData(paramsHandler);
                if (compContext.Beam_Part == null)
                {
                    var beamCutter = new BeamCutCreator(compContext.MainPart, compContext.SubPart, compContext.Comp, paraData1);
                    beamCutter.Execute(compContext.AlignPlane);
                }
                else
                {
                    var beamCutter1 = new BeamCutCreator(compContext.Beam_Part, compContext.SubPart, compContext.Comp, paraData1);
                    beamCutter1.Execute(compContext.AlignPlane);
                }
                //槽口示例
                NotchHandler notchHandler = new NotchHandler(compContext, paramsHandler);
                notchHandler.Execute();
                //焊缝示例
                WeldHandler weldHandler = new WeldHandler(compContext, paramsHandler);
                weldHandler.Execute();
            }
            return true;
        }


        /// <summary>
        /// 显示组件的属性窗，必须实现此方法
        /// </summary>
        /// <param name="ownerForm">组件窗口所属主窗口</param>
        /// <returns>是否显示成功</returns>
        /// <remarks><b>其他组件可直接复制此方法</b></remarks>
        public override bool OnShowPluginPropsForm(Form ownerForm)
        {
#if false
            if (m_propsForm == null) {
                m_propsForm = new Form_PluginMain(this);
                m_propsForm.Owner = ownerForm;
            }

            m_propsForm.ShowPluginProps();
#else
            Form_ComponentProps pluginForm = GetMainForm(ownerForm);
            pluginForm.ShowPluginProps();
#endif
            return true;
        }

        /// <summary>
        /// 显示组件实例的属性窗，必须实现此方法
        /// </summary>
        /// <param name="instObj">组件实例对象</param>
        /// <param name="defId">组件默认值ID</param>
        /// <param name="ownerForm">组件窗口所属主窗口</param>
        /// <returns>是否显示成功</returns>
        /// <remarks><b>其他组件可直接复制此方法</b></remarks>
        public override bool OnShowInstancePropsForm(ComponentBase instObj, long defId, Form ownerForm)
        {
            if (instObj == null)
                return false;

#if false
            if (m_propsForm == null) {
                m_propsForm = new Form_PluginMain(this);
                m_propsForm.Owner = ownerForm;
            }

            m_propsForm.ShowInstanceProps(instObj,
                new ParamsHandler().InitializeDefaultValues(this.GetDefaultsData(defId)));
            return true;
#else
            Form_ComponentProps pluginForm = GetMainForm(ownerForm);
            pluginForm.ShowInstanceProps(instObj,
                new ParamsHandler().InitializeDefaultValues(GetDefaultsData(defId)));
            return true;
#endif
        }

        private Form_PluginMain GetMainForm(Form ownerForm)
        {
            Form_PluginMain pluginForm = FormPool.GetPluginForm(Guid, "main") as Form_PluginMain;
            if (pluginForm == null)
            {
                pluginForm = new Form_PluginMain(this);
                pluginForm.Owner = ownerForm;
                FormPool.PushPluginForm(Guid, "main", pluginForm);
            }
            return pluginForm;
        }
    }
}



