﻿using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace Example_ConnectionComponent1
{

    /// <summary>
    /// 节点组件通用主窗口，其他组件可参考此类实现其主窗口
    /// </summary>
    public partial class Form_PluginMain : LoongCAD.UI.Component.Form_ConnectionComponentProps
    {

        private readonly DObject m_weldProps = new DObject();
        private readonly ComponentPlugin m_refPlugin = null;
        private Form_WeldParameter m_weldForm = null;
        private readonly DObject m_rootUIProps = new DObject();

        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            OnPostIntialize();
        }

        public override ComponentPlugin ReferencePlugin => m_refPlugin;

        private void OnPostIntialize()
        {
            // 设置窗口标题
            Text = "节点(000)";
            // 若使用对象池，需要设置此属性为false
            DisposeOnClose = false;
            // 螺栓标准变化之后调用螺栓内容的自适应方法
            panel_bolt_basic_props.BoltStandardChanged += panel_bolt_content_props.AdaptBoltContents;
            foreach (TabPage curPage in tab_main.TabPages)
            {
                curPage.AutoScroll = true;
            }
            AddItemForKevValve(cmb_bolt_x_base, new List<string>() { "默认", "", "左", "中", "右", "腹板" });
            AddItemForKevValve(cmb_bolt_y_base, new List<string>() { "默认", "", "顶", "中", "下" });
        }
        /// <summary>
        /// 设置当前窗口的属性值
        /// </summary>
        /// <param name="paras"></param>
        /// <param name="root"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if (root == null)
            {
                m_weldProps.Clear();
                m_weldProps.CopyFrom(paras);
            }
        }
        /// <summary>
        /// 获取当前窗口的属性值
        /// </summary>
        /// <param name="root"></param>
        /// <param name="exclude"></param>
        /// <param name="bConsiderCheck"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        /// <returns></returns>
        public override DObject GetParameters(Control root = null, List<Control> exclude = null, bool bConsiderCheck = true)
        {
            DObject retProps = new DObject();
            if (!bConsiderCheck || act_weld_props.Checked)
            {
                retProps.CopyFrom(m_weldProps);
            }
            DObject baseProps = base.GetParameters(root, exclude, true);
            retProps.CopyFrom(baseProps);
            return retProps;
        }

        /// <summary>
        /// 当前组件需要重写此方法，并刷新螺栓基础面板
        /// </summary>
        /// <param name="root"></param>
        public override void ClearUI(Control root = null)
        {
            // 刷新螺栓基础面板
            panel_bolt_basic_props.RefreshControls();
            base.ClearUI(root);
        }

        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            SetPlaceholders(defaultValues);
        }

        private void btn_weld_props_Click(object sender, EventArgs e)
        {
            if (m_weldForm == null)
            {
                m_weldForm = new Form_WeldParameter();
                m_weldForm.Owner = Owner;
                m_weldForm.ApplyHandler += ApplyWeldProps;
                m_weldForm.ModifyHandler += ModifyWeldProps;
                m_weldForm.GetHandler += GetWeldProps;
            }

            m_weldForm.Show(m_weldProps, null);
        }

        protected override void OnClickCheckBox(object sender, EventArgs e)
        {
            System.Windows.Forms.CheckBox[] userChecks = new System.Windows.Forms.CheckBox[] { act_weld_props };
            SwitchControlChecks(null, userChecks);
        }

        /// <summary>
        /// 释放当前组件的焊缝窗口
        /// </summary>
        /// <param name="e">窗口释放事件</param>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected override void OnFormClosing(FormClosingEventArgs e)
        {
            if (m_weldForm != null)
            {
                m_weldForm.ApplyHandler -= ApplyWeldProps;
                m_weldForm.ModifyHandler -= ModifyWeldProps;
                m_weldForm.GetHandler -= GetWeldProps;
                m_weldForm.Dispose();
                m_weldForm = null;
            }
            base.OnFormClosing(e);
        }

        /// <summary>
        /// 应用焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ApplyWeldProps(DObject srcProps)
        {
            m_weldProps.CopyFrom(srcProps);
        }

        /// <summary>
        /// 修改焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ModifyWeldProps(DObject srcProps)
        {
            ModifyFocusedProps(srcProps);
        }

        /// <summary>
        /// 获取焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected DObject GetWeldProps()
        {
            var refPlugin = ReferencePlugin;
            if (refPlugin == null)
                return m_weldProps;
            var curScene = LoongCAD.STC.STC.Document.FocusedScene;
            if (curScene == null)
                return m_weldProps;
            System.Guid pluginGuid = refPlugin.Guid;
            LoongCAD.STC.Object3D.ObjectBase[] curObjs = curScene.GetSelectedObjects(
                obj => obj is LoongCAD.STC.Object3D.ComponentBase &&
                ((LoongCAD.STC.Object3D.ComponentBase)obj).CompGuid == pluginGuid);
            if (curObjs == null || curObjs.Length == 0)
                return m_weldProps;
            LoongCAD.STC.Object3D.ComponentBase comp = curObjs[0] as LoongCAD.STC.Object3D.ComponentBase;
            if (comp == null)
                return m_weldProps;

            return comp.UIProps;
        }

        private void AddItemForKevValve(KeyValueComboBox ui, List<string> cfg)
        {
            ui.ClearItems();
            for (int i = 0; i < cfg.Count; i++)
            {
                ui.AddItem(i, cfg[i]);
            }
            ui.UpdateItems();
        }
        private void angle_enable(object sender, EventArgs e)
        {
            var data = sender as System.Windows.Forms.ComboBox;
            if (data.SelectedIndex == 1)
            {
                stiff_angle_x.ReadOnly = true;
                stiff_angle_y.ReadOnly = true;
                stiff_angle_x.Enabled = false;
                stiff_angle_y.Enabled = false;
            }
            else if (data.SelectedIndex == 3 || data.SelectedIndex == 4)
            {
                stiff_angle_x.ReadOnly = false;
                stiff_angle_y.ReadOnly = true;
                stiff_angle_x.Enabled = true;
                stiff_angle_y.Enabled = false;
            }
            else
            {
                stiff_angle_x.ReadOnly = false;
                stiff_angle_y.ReadOnly = false;
                stiff_angle_x.Enabled = true;
                stiff_angle_y.Enabled = true;
            }
        }
    }
}
