﻿using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.Util;
using System;

namespace Example_ConnectionComponent1
{
    public partial class Form_WeldParameter : ParamFormBase
    {

        public delegate void OnApplyPropsDelegate(DObject srcProps);
        public delegate void OnModifyPropsDelegate(DObject srcProps);
        public delegate DObject OnGetPropsDelegate();

        private OnApplyPropsDelegate m_applyHandler = null;
        private OnGetPropsDelegate m_getHandler = null;
        private OnModifyPropsDelegate m_modifyHandler = null;

        public Form_WeldParameter()
        {
            InitializeComponent();
            OnPostIntialize();
        }

        private void OnPostIntialize()
        {
            // 窗口关闭时不释放实例，只是隐藏窗口
            // 此行代码很重要
            DisposeOnClose = false;
            weldbar.OkButton.Click += OnClickOk;
            weldbar.ApplyButton.Click += OnClickApply;
            weldbar.ModifyButton.Click += OnClickModify;
            weldbar.GetButton.Click += OnClickGet;
            weldbar.CancelButton.Click += OnClickCancel;
        }

        public void Show(DObject srcProps, DObject defaultValues)
        {
            ClearUI();
            SetParameters(srcProps);
            EnableControlChecks(true);
#if false
            this.UpdatePlaceholders(srcProps, defaultValues);
#else
            ClearPlaceholders();
#endif
            Show();
        }

        public OnApplyPropsDelegate ApplyHandler
        {
            set => m_applyHandler = value;
            get => m_applyHandler;
        }

        public OnGetPropsDelegate GetHandler
        {
            set => m_getHandler = value;
            get => m_getHandler;
        }

        public OnModifyPropsDelegate ModifyHandler
        {
            set => m_modifyHandler = value;
            get => m_modifyHandler;
        }

        private void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            DObject placeholders = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            ClearPlaceholders();
            SetPlaceholders(placeholders);
        }

        private void OnClickOk(object sender, EventArgs e)
        {
            if (m_applyHandler != null)
            {
                m_applyHandler(GetParameters());
            }
            Hide();
        }

        private void OnClickApply(object sender, EventArgs e)
        {
            if (m_applyHandler != null)
            {
                m_applyHandler(GetParameters());
            }
        }

        private void OnClickModify(object sender, EventArgs e)
        {
            if (m_modifyHandler != null)
            {
                m_modifyHandler(GetParameters());
            }
        }

        private void OnClickGet(object sender, EventArgs e)
        {
            if (m_getHandler != null)
            {
                var srcProps = m_getHandler();
                SetParameters(srcProps);
            }
        }

        private void OnClickCheckBox(object sender, EventArgs e)
        {
            SwitchControlChecks();
        }

        private void OnClickCancel(object sender, EventArgs e)
        {
            Hide();
        }

    }
}
