﻿using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Plugins;

namespace Example_ConnectionComponent1.Handlers
{
    public class BeamHandler  //该部分为创建直梁的示例代码，其中梁切割的示例代码也在该类
    {
        private readonly ComponentInstanceContext m_context = null;  //组件公用参数
        private readonly ParamsHandler m_paramsHandler = null;     //默认值和UI存储参数
        //
        private readonly BeamProperties beamProperties = new BeamProperties();  //直梁截面
        private readonly PlateProperties beam_props = new PlateProperties();   //直梁参数
        public BeamHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            //初始化参数
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public void Execute()
        {
            //初始化参数
            InitParams();
            //创建直梁示例代码:
            {
                //1.设置控制点
                Vec3 point1 = m_context.SubPlane_top.Intersect(m_context.ProjectPlane, m_context.AlignPlane);
                Vec3 point2 = m_context.SubPlane_bottom.Intersect(m_context.ProjectPlane, m_context.AlignPlane);
                //2.设置截面(如果UI界面没有输入截面，则用默认截面)
                if (beamProperties.Profile == "")
                    beamProperties.Profile = "PL" + beam_props.Thickness.ToString() + "*" + m_context.SubPart.Profile.ShapeWidth.ToString();
                //3.创建直梁
                var beam_part = m_context.Comp.GetOrCreateSteelBeam("beam_part", beamProperties.Profile, point1, point2);
                m_context.Beam_Part = beam_part;
                //4.设置直梁参数
                beam_props.ApplyToPart(beam_part);
                Vec3 vx = m_context.MainPlane_top.Normal;
                Vec3 vz = point1 - point2;
                Vec3 vy = vx.Cross(vz);
                beam_part.WorkAxis = new Mat43(vx, vy, vz, Vec3.ZERO);  //设置工作坐标系
                beam_part.HorizonOffsetSide = Define.ObjectSide.SIDE_RIGHT;
                beam_part.HorizonOffset = 0;
                beam_part.RotationSide = Define.ObjectSide.SIDE_TOP;
                beam_part.Rotation = 0;
                beam_part.VertOffsetSide = Define.ObjectSide.SIDE_MIDDLE;
                beam_part.VerticalOffset = 0;
                beam_part.Refresh();
            }
            //创建梁切割示例代码:
            {
                //1.设置控制点
                Vec3 point1 = m_context.SubPlane_top.Intersect(m_context.ProjectPlane, m_context.AlignPlane.Move(200));
                Vec3 point2 = m_context.SubPlane_bottom.Intersect(m_context.ProjectPlane, m_context.AlignPlane.Move(200));
                //2.设置截面(默认截面为圆截面)
                string profile = "D" + m_context.SubPart.Profile.ShapeWidth.ToString();
                //3.创造切割体
                m_context.Comp.CreateBeamCutter(m_context.SubPart, profile, point1, point2, true);
            }
        }
        private void InitParams()
        {
            //获取默认值或UI界面输入值
            beamProperties.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.BEAM_PROFILE));
            beam_props.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.BEAM_PROPS));
        }
    }
}
