﻿using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.Util;

namespace Example_ConnectionComponent1
{ 
    public class BoltOrHoleHandler
    {
        private readonly ComponentInstanceContext m_context = null;
        private readonly ParamsHandler m_paramsHandler = null;
        /// <summary>
        ///螺栓参数
        /// </summary>
        private double[] m_boltXIntervals;
        private double[] bolt_xintervals;
        private double[] m_boltYIntervals;
        private double[] bolt_yintervals;
        private int bolt_lie;
        private int bolt_hang;
        private int m_boltPosType;
        private readonly BoltParameter m_boltParam = new BoltParameter();
        private readonly uint m_sectFlags = 0;
        public BoltOrHoleHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public void Execute()
        {
            //初始化(螺栓参数的初始化也在该函数中完成)
            InitParams();
            //创建螺栓示例代码:
            {
                //1.设置螺栓的起点和终点
                Vec3 st = m_context.Beam_Part.CtrlPoints[0].WorldPos / 2 + m_context.Beam_Part.CtrlPoints[1].WorldPos / 2;  //起点从直梁的中间开始
                Vec3 ed = st - m_context.Normal_top * 10;
                //2.创建
                BoltGroup bolt_group = m_context.Comp.GetOrCreateBoltGroup("bolt_group", m_context.MainPart, m_context.Beam_Part, st, ed);
                //3.设置参数
                bolt_group.WorkAxis = new Mat43(-m_context.Normal_front, m_context.Normal_right, m_context.MainPlanes.HeadBound.Normal, Vec3.ZERO);
                bolt_group.RotationSide = Define.ObjectSide.SIDE_BACK;
                bolt_group.Rotation = 0;
                m_boltParam.ApplyToBoltGroup(bolt_group);
            }
            //创建洞示例代码:
            {//洞其实是没有创建螺母和垫板的螺栓,创建方法和创建螺栓相同，只是所用函数不同
                //1.设置起点和终点
                Vec3 st = m_context.Beam_Part.CtrlPoints[0].WorldPos / 2 + m_context.Beam_Part.CtrlPoints[1].WorldPos / 2 + m_context.Normal_top * 30;
                Vec3 ed = st - m_context.Normal_top * 10;
                //2.创建
                BoltGroup hole = m_context.Comp.GetOrCreateBoltHoles("hole", null, m_context.Beam_Part, st, ed);
                //3.设置参数
                hole.WorkAxis = new Mat43(-m_context.Normal_front, m_context.Normal_right, m_context.MainPlanes.HeadBound.Normal, Vec3.ZERO);
                hole.RotationSide = Define.ObjectSide.SIDE_BACK;
                hole.Rotation = 0;
                hole.BoltSize = 30;
                hole.Refresh();
            }

        }
        private void InitParams()
        {
            //螺栓参数
            m_boltXIntervals = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_X_INTERVALS);
            bolt_lie = m_paramsHandler.GetIntValue(ParamNames.BOLT_LIE);
            bolt_hang = m_paramsHandler.GetIntValue(ParamNames.BOLT_HANG);
            m_boltYIntervals = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_Y_INTERVALS);
            m_boltPosType = m_paramsHandler.GetIntValue(ParamNames.BOLT_POS_TYPE);
            DObject basicProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_BASIC_PROPS);
            DObject holeProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_HOLE_PROPS);
            DObject contentProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_CONTENT_PROPS);
            DObject extendProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_EXTEND_PROPS);
            m_boltParam.CopyFromBasicProps(basicProps);
            m_boltParam.CopyFromHoleProps(holeProps);
            m_boltParam.CopyFromContentProps(contentProps);
            m_boltParam.CopyFromExtendProps(extendProps);
            //设置螺栓排布(一般直接复制到代码即可)
            switch (m_boltPosType)
            {
                case 2:
                    m_boltParam.BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_X_ODD;
                    break;
                case 3:
                    m_boltParam.BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_X_EVEN;
                    break;
                case 4:
                    m_boltParam.BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_Y_ODD;
                    break;
                case 5:
                    m_boltParam.BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_Y_EVEN;
                    break;
                case 1:
                default:
                    m_boltParam.BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ARRAY;
                    break;
            }
            //设置螺栓的开孔(一般直接复制到代码即可)
            {
                switch (m_boltParam.SlotIn)
                {
                    case BoltParameter.SoltInMode.BEAM:
                        m_boltParam.SectFlags = m_sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE0;
                        break;
                    case BoltParameter.SoltInMode.PLATE:
                        {
                            m_boltParam.SectFlags = m_sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE1;
                            m_boltParam.SectFlags = m_sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE2;
                        }
                        break;
                    case BoltParameter.SoltInMode.BOTH:
                        {
                            m_boltParam.SectFlags = m_sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE0;
                            m_boltParam.SectFlags = m_sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE1;
                            m_boltParam.SectFlags = m_sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE2;
                        }
                        break;
                    default:
                        break;
                }
                //设置螺栓的垫片和螺母(一般直接复制到代码即可)
                if (m_boltParam.PartWasherUp)
                {
                    m_boltParam.SectFlags = m_boltParam.SectFlags | Define.BOLT_SECT_WASHER_UP;
                }
                if (m_boltParam.PartWasherDown0)
                {
                    m_boltParam.SectFlags = m_boltParam.SectFlags | Define.BOLT_SECT_WASHER_DOWN0;
                }
                if (m_boltParam.PartWasherDown1)
                {
                    m_boltParam.SectFlags = m_boltParam.SectFlags | Define.BOLT_SECT_WASHER_DOWN1;
                }
                if (m_boltParam.PartNut0)
                {
                    m_boltParam.SectFlags = m_boltParam.SectFlags | Define.BOLT_SECT_NUT0;
                }
                if (m_boltParam.PartNut1)
                {
                    m_boltParam.SectFlags = m_boltParam.SectFlags | Define.BOLT_SECT_NUT1;
                }
            }
            //设置螺栓的间距(一般情况不同,代码不同)
            if (m_boltYIntervals.Length == 0)
                m_boltYIntervals = new double[] { 60.0 };
            if (m_boltXIntervals.Length == 0)
                m_boltXIntervals = new double[] { 80 };
            if (bolt_hang == int.MaxValue)
                bolt_hang = m_boltYIntervals.Length + 1;
            if (bolt_lie == int.MaxValue)
                bolt_lie = m_boltXIntervals.Length + 1;
            bolt_yintervals = new double[bolt_hang - 1];
            bolt_xintervals = new double[bolt_lie - 1];
            for (int i = 0, j = 0; i < bolt_hang - 1; i++)
            {
                bolt_yintervals[i] = m_boltYIntervals[j];
                if (j < m_boltYIntervals.Length - 1)
                    j++;
                else j = 0;
            }
            for (int i = 0, j = 0; i < bolt_lie - 1; i++)
            {

                bolt_xintervals[i] = m_boltXIntervals[j];
                if (j < m_boltXIntervals.Length - 1)
                    j++;
                else j = 0;
            }
            m_boltParam.Bolt_Xintervals = bolt_yintervals;
            m_boltParam.Bolt_Yintervals = bolt_xintervals;
        }
    }
}
