﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;

namespace Example_ConnectionComponent1
{
    public class FittingHandler
    {
        private readonly ComponentInstanceContext m_context = null;
        private readonly ParamsHandler m_paramsHandler = null;
        //
        private readonly PlateProperties beam_props = new PlateProperties();   //直梁截面参数
        public FittingHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public void Execute()
        {
            //接合面示例代码: 
            {
                //1.初始化参数
                InitParams();
                //2.设置接合面(接合面为主梁包围面上靠近次梁的那个面，且移动直梁的厚度以防止直梁嵌入主次梁)
                m_context.AlignPlane = m_context.MainPlane_top.Move(beam_props.Thickness).Clone();
                //3.接合次梁
                m_context.Comp.CreateAlignPlane(m_context.SubPart, m_context.AlignPlane, true);
            }
            //切割面示例代码:
            {
                //1.设置切割面(本节点为切割次梁尾部，无实际作用,具体要以实际来进行切割)
                Plane cutter_plane = new Plane(m_context.AlignPlane.Move(800).Origin, -m_context.AlignPlane.Normal);
                //2.切割
                m_context.Comp.CreateCutPlane(m_context.SubPart, cutter_plane, true);
            }
        }
        /// <summary>
        /// 初始化参数
        /// </summary>
        private void InitParams()
        {
            beam_props.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.BEAM_PROPS));
        }
    }
}
